# ---+ Extensions
# ---++ ShibLDAP 
# This is the configuration used by <b>ShibLdapContrib</b>, a derivative of LdapContrib
# that is useful for any Foswiki setups that are relying on SAML federated authentication
# for authenticating their users.
# <p>
# This uses Shibboleth (SAML) for login, but still can use an LDAP server for group information,
# as well as groups provided through Shibboleth (as values of the configured attribute below).
# Use this in conjunction with the <b>ShibLdapApacheLogin</b> Login Manager, and 
# the <b>ShibLdapUserMapping</b> Users Mapping Manager.
# (see the Security Setting section)
#
# Note that this variation of LdapContrib still retains the Cache file from that, for
# caching information read from a LDAP server. But that, given one is relying on identities
# coming from Shib, it only makes sense to cache Group information, and to use Group objects
# that contain a member-type attribute that is <b>NOT</b> a DN, because these users
# won't/don't need to have a "local identity" in that LDAP server. And the loginName and
# group membership(s) delivered by Shib will not be written into that Cache, because:
#  - they'd get deleted as soon as the Cache is refreshed the next time
#  - it would cause writes to the Cache with almost every new user interaction
#    with Foswiki, greatly raising the possibility of clashes
#  - this will perform better by not trying to add the Shib info into the cache
#  - the Shib info is in the environment with every interaction 

# ---+++ Debug setting (get a number of debug messages into log)
# **BOOLEAN**
# Enable/disable debug output to the Foswiki '../working/logs/debug.log' file
$Foswiki::cfg{ShibLdap}{Debug} = 0;

# ---+++ Shibboleth 
# This is the configuration specific to Shibboleth,
# assuming there is a Shibboleth SP configured to place
# attributes passed from the user's IdP into environment variables.

# **STRING**
# Environment variable (Shib-provided attribute) to read user login name from.
$Foswiki::cfg{ShibLdap}{ShibLoginEnvironmentVariable} = 'REMOTE_USER';

# **STRING**
# Environment variable (Shib-provided attribute) to read user email attribute from
$Foswiki::cfg{ShibLdap}{ShibMailAttribute} = 'mail';

# **STRING**
# The Environment variable(s) (Shib-provided attribute(s)) from which the user's WikiName should
# be constructed. This can be a comma-separated list of attributes which will be combined in
# that order (with a space separating each), and to which then any additional processing such as
# normalization, rewriting, etc. will then be applied.
$Foswiki::cfg{ShibLdap}{ShibWikiNameAttributes} = 'eduPersonPrincipalName';

# **BOOLEAN**
# Enable use of Shib-provided groups. If you switch this off the other Shib group-related settings
# have no effect. This flag is of use if you DON'T want to use groups provided by Shib,
# but still want to rely on Shib for login processing.
$Foswiki::cfg{ShibLdap}{ShibMapGroups} = 1;

# **STRING**
# This is the name of the Environment variable (Shib-provided attribute) that holds the names of
# the Shib-provided groups for the person -- groups in which this person should be considered to
# to be a member.
$Foswiki::cfg{ShibLdap}{ShibGroupAttribute} = 'isMemberOf';

# **BOOLEAN**
# Flag indicating if any Shib-delivered groups that get mapped to the same name as an already
# cached (pre-caching) LDAP group should be "merged". 
# I.e. if a person's IdP (or at least the full set of attributes provided by the SP) 
# delivers a group name that matches an "already provisioned locally-sourced"
# LDAP group, do you want to allow that IdP/SP-asserted value to be "trusted"
# and add this person to that group for purposes of authorization, or not?
$Foswiki::cfg{ShibLdap}{ShibMergeWithLdapGroups} = 1;

# **BOOLEAN**
# Enable/disable normalization of the group names as they come from Shib. Normalizing means
# applying "standard wikiname formatting rules".
$Foswiki::cfg{ShibLdap}{ShibNormalizeGroupNames} = 1;

# **PERL**
# Configuration that allows flexible rewrite rules for Shib-delivered group names.
# The formatting/processing here is exactly the same as the rewrite rule configuration
# that LdapContrib provides for group names that are read in from an LDAP server.
# This is a hash mapping of rewrite rules. Rules are separated by commas. A rule has 
# the form:
# <pre>{
#   'pattern1' => 'substitute1', 
#   'pattern2' => 'substitute2' 
# }</pre>
# consists of a name pattern that has to match the group name to be rewritten
# and a substitute value that is used to replace the matched pattern. The
# substitute might contain $1, $2, ... , $5 to insert the first, second, ..., fifth
# bracket pair in the key pattern. (see perl manual for regular expressions).
# Example: '(.*)_group' => '$1'
$Foswiki::cfg{ShibLdap}{ShibRewriteGroupNames} = {
};

# ---+++ Shibboleth-applicable settings that are brought forward from LdapContrib 
# (and would still impact LDAP-based person objects if any such involved).
# This is the configuration that applies to Shibboleth-provided values
# as appropriate, but are "re-used" from the original LdapContrib from
# which this is derived. Really, if using this module, the only login names
# and person wiki names to be "normalized" etc. are coming from Shib, not 
# from LDAP. However, if for some reason you were still caching person
# objects from an LDAP server, these settings would also apply to the person
# login names and their wikinames.

# **BOOLEAN**
# Enable/disable normalization of WikiUserNames as they come from Shib (or LDAP).
# If the WikiNameAttribute is set to 'mail' a trailing @my.domain.com
# is stripped. WARNING: if you switch this off you have to guarantee that the WikiNames
# in the WikiNameAttribute are a proper WikiWord (camel-case, no spaces, no umlauts etc).
$Foswiki::cfg{ShibLdap}{NormalizeWikiNames} = 1;

# **BOOLEAN**
# Enable/disable normalization of login names as they come from Shib (or LDAP).
$Foswiki::cfg{ShibLdap}{NormalizeLoginNames} = 0;

# **BOOLEAN**
# Enable/disable case sensitive login names. If disabled case doesn't matter logging in.
$Foswiki::cfg{ShibLdap}{CaseSensitiveLogin} = 0;

# **STRING**
# Alias old !WikiNames to new account. This is a comma separated list of
# "OldName=NewName" values.
# Warning: this setting is deprecated - use <code>RewriteWikiNames</code> instead
$Foswiki::cfg{ShibLdap}{WikiNameAliases} = '';

# **PERL**
# Rewrite rules for User WikiNames from Shib or LDAP.
# A hash mapping of rewrite rules. Rules are separated by commas. A rule has 
# the form 
# <pre>{
#   'pattern1' => 'substitute1', 
#   'pattern2' => 'substitute2' 
# }</pre>
# consists of a name pattern that has to match the wiki name to be rewritten
# and a substitute value that is used to replace the matched pattern. The
# substitute might contain $1, $2, ... , $5 to insert the first, second, ..., fifth
# bracket pair in the key pattern. (see perl manual for regular expressions).
# Example: '(.*)_users' => '$1'
$Foswiki::cfg{ShibLdap}{RewriteWikiNames} = {
  '^(.*)@.*$' => '$1'
};

# ---+++ LDAP Connection settings

# **STRING**
# IP address (or hostname) of the LDAP server
$Foswiki::cfg{ShibLdap}{Host} = 'ldap.my.domain.com';

# **NUMBER**
# Port used when binding to the LDAP server
$Foswiki::cfg{ShibLdap}{Port} = 389;

# **NUMBER**
# Ldap protocol version to use when querying the server; 
# Possible values are: 2, 3
$Foswiki::cfg{ShibLdap}{Version} = '3';

# **STRING**
# Base DN to use in searches
$Foswiki::cfg{ShibLdap}{Base} = 'dc=my,dc=domain,dc=com';

# **STRING**
# The DN to use when binding to the LDAP server; if undefined anonymous binding
# will be used. Example 'cn=proxyuser,dc=my,dc=domain,dc=com'
$Foswiki::cfg{ShibLdap}{BindDN} = '';

# **PASSWORD**
# The password used when binding to the LDAP server
$Foswiki::cfg{ShibLdap}{BindPassword} = 'secret';

# **STRING**
# Set this to the charset encoding of data coming from the LDAP server.
# Normally this should be 'utf-8', but might differ in some cases.
# Data read from the server will then be converted from this encoding
# and translated to your site's charset encoding as configured in <code>{Site}{CharSet}</code>.
# WARNING: if you change the charset encoding after having used a different one for some time
# you will require to remove all cached data in <code>.../working/work_areas/LdapContrib</code>
# and <code>.../working/work_areas/LdapNgPlugin/cache</code>. Otherwise this data 
# will be reused assuming a false charset encoding.
$Foswiki::cfg{ShibLdap}{CharSet} = 'utf-8';

# **BOOLEAN**
# Use SASL authentication when binding to the server; Note, when using SASL the 
# BindDN and BindPassword setting are used to configure the SASL access.
$Foswiki::cfg{ShibLdap}{UseSASL} = 0;

# **STRING**
# List of SASL authentication mechanism to try; defaults to 'PLAIN CRAM-MD5
# EXTERNAL ANONYMOUS'
$Foswiki::cfg{ShibLdap}{SASLMechanism} = 'PLAIN CRAM-MD5 EXTERNAL ANONYMOUS';

# **BOOLEAN**
# Use Transort Layer Security (TLS) to encrypt the connection to the LDAP server.
# You will need to specify the servers CA File using the TLSCAFile option
$Foswiki::cfg{ShibLdap}{UseTLS} = 0;

# **STRING**
# This defines the version of the SSL/TLS protocol to use. Possible values are:
# 'sslv2', 'sslv3',  'sslv2/3' or 'tlsv1'
$Foswiki::cfg{ShibLdap}{TLSSSLVersion} = 'tlsv1';

# **STRING**
# Specify how to verify the servers certificate. Possible values are: 'require', 'optional'
# or 'require'.
$Foswiki::cfg{ShibLdap}{TLSVerify} = 'require';

# **STRING**
# Pathname of the directory containing CA certificates
$Foswiki::cfg{ShibLdap}{TLSCAPath} = '';

# **STRING**
# Filename containing the certificate of the CA which signed the server’s certificate.
$Foswiki::cfg{ShibLdap}{TLSCAFile} = '';

# **STRING**
# Client side certificate file
$Foswiki::cfg{ShibLdap}{TLSClientCert} = '';

# **STRING**
# Client side private key file
$Foswiki::cfg{ShibLdap}{TLSClientKey} = '';


# ---+++ LDAP-based Group settings
# The settings below configure the use of LDAP groups, and how/if the group names
# are normalized, rewritten, etc.

# **BOOLEAN**
# Enable use of LDAP groups. If you switch this off the group-related settings
# have no effect. This flag is of use if you don't want to define groups in LDAP
# but still want to map LoginNames to WikiNames on the base of LDAP data.
$Foswiki::cfg{ShibLdap}{MapGroups} = 1;

# **PERL**
# A list of trees where to search for group records.
$Foswiki::cfg{ShibLdap}{GroupBase} = ['ou=group,dc=my,dc=domain,dc=com'];

# **STRING**
# Filter to be used to find groups. Compare to LoginFilter. Remember, you are wanting
# to maintain and use a set of groups that will be useful in matching up with users
# whose identities are provided by Shib, and are not (or at least not required to be)
# contained within the local LDAP as person objects.
$Foswiki::cfg{ShibLdap}{GroupFilter} = 'objectClass=groupOfNames';

# **SELECT sub,one**
# The scope of the search for groups starting at GroupBase. While "sub" search recursively
# a "one" will only search up to one level under the GroupBase.
$Foswiki::cfg{ShibLdap}{GroupScope} = 'sub';

# **STRING**
# This is the name of the attribute that holds the name of the 
# group in a group record.
$Foswiki::cfg{ShibLdap}{GroupAttribute} = 'description';

# **STRING**
# The attribute that should be used to collect group members. This is the name of the
# attribute in a group record used to point to the user record. For example, in a posix setting this
# is the uid of the relevant posixAccount. If groups are implemented using the object class
# 'groupOfNames' the 'member' attribute stores a literal DN pointing to the account record. In this
# case you have to switch on the MemberIndirection flag below. But a common extension to groupOfNames
# is to add a 'hasMember' attribute, which *does not* need to be DN, and would likely be much more
# useful in conjunction with the functionality/purpose of this ShibLdapContrib extension.
#
# Most likely, since you are using these LDAP groups *not* with identities (and thus DNs) from LDAP,
# but instead wanting to list the identities (login names) of users coming from Shib, you will want to
# use Group objects which have an 'hasMember' of attribute, which allows for any user identifier.
$Foswiki::cfg{ShibLdap}{MemberAttribute} = 'hasMember';

# **STRING**
# This is the name of the attribute in a group record used to point to the inner group record.
# This value is often the same than MemberAttribute but may differ for some LDAP servers.
$Foswiki::cfg{ShibLdap}{InnerGroupAttribute} = '';

# **BOOLEAN**
# Flag indicating wether the MemberAttribute of a group stores a DN. (Unlikely you
# will find such a group useful when using Shib-provided identities.)
$Foswiki::cfg{ShibLdap}{MemberIndirection} = 0;

# **BOOLEAN**
# Enable/disable normalization of group names as they come from LDAP:
$Foswiki::cfg{ShibLdap}{NormalizeGroupNames} = 0;

# **PERL**
# LDAP-based group name rewrite rules, as opposed to those above for
# Shib-provided group name rewriting.
# A hash mapping of rewrite rules. Rules are separated by commas. A rule has 
# the form 
# <pre>{
#   'pattern1' => 'substitute1', 
#   'pattern2' => 'substitute2' 
# }</pre>
# consists of a name pattern that has to match the group name to be rewritten
# and a substitute value that is used to replace the matched pattern. The
# substitute might contain $1, $2, ... , $5 to insert the first, second, ..., fifth
# bracket pair in the key pattern. (see perl manual for regular expressions).
# Example: '(.*)_users' => '$1'
$Foswiki::cfg{ShibLdap}{RewriteGroups} = {
};

# **BOOLEAN**
# Flag indicating if LDAP groups that get the same name are merged. For example, given two 
# LDAP groups that end up having the same name even though they have a different distinguished name
# or have been rewritten to match on the same group name (see RewriteGroups), then members
# of both groups are merged into one group of that name.
$Foswiki::cfg{ShibLdap}{MergeGroups} = 0;

# **BOOLEAN**
# Flag indicating whether we fallback to WikiGroups. If this is switched on, 
# standard Wiki groups will be used as a fallback if a group definition of a given
# name was not found in the LDAP database. Not entirely sure if this will still
# work correctly with this Shib-based module, which is focused on being able to
# combine groups from Shib and LDAP, and not really use WikiGroups at all. I.e trying
# to also use WikiGroups in addition to Shib and LDAP groups has not been tested,
# although nothing was explicitly done to try and "disable this from working".
$Foswiki::cfg{ShibLdap}{WikiGroupsBackoff} = 0;

# **STRING**
# Again, with this Shib-based connector, it is not expected that you will
# use LDAP-based person objects, and thus you won't need/use any group-membership
# attribute from the person object. But in case there is some need for that, this
# configuration is carried over from LdapContrib.
# This is the name of the attribute that holds the primary group attribute.
# This attribute is stored as part of the user record and refers to the
# primary group this user is in. Sometimes, this membership is not captured
# in the group record itself but in the user record to make it the primary group
# a user is in.
$Foswiki::cfg{ShibLdap}{PrimaryGroupAttribute} = 'gidNumber';


# ---+++ LDAP "person object" (User) settings
# The options below configure how the wiki will extract account records from LDAP.
# Note that the expectation with this module is that you <b>won't</b> actually want
# to read/cache person records from LDAP, just Group objects.
 
# **PERL**
# A list of trees where to search for users records. A typical-type value for this,
# if you were going to bother/need to read in person entries from an LDAP server,
# would be something like 'ou=people,dc=my,dc=domain,dc=com'.
$Foswiki::cfg{ShibLdap}{UserBase} = [];

# **STRING**
# Filter to be used to find login accounts. Compare to GroupFilter below
$Foswiki::cfg{ShibLdap}{LoginFilter} = 'objectClass=posixAccount';

# **SELECT sub,one**
# The scope of the search for users starting at UserBase. While "sub" search recursively
# a "one" will only search up to one level under the UserBase.
$Foswiki::cfg{ShibLdap}{UserScope} = 'sub';

# **STRING**
# The user login name attribute. This is the attribute name that is
# used to login. Presumably, if you were actually going to still use any
# person information from a local LDAP to complement the information about the user
# provided by Shib, you'd match the Shib-provided identity (login name) to
# this LDAP attribute to find a the matching LDAP entry.
$Foswiki::cfg{ShibLdap}{LoginAttribute} = 'uid';

# **STRING**
# The user mail attribute. This is the attribute name used to fetch
# users e-mail.
$Foswiki::cfg{ShibLdap}{MailAttribute} = 'mail';

# **STRING**
# The user's wiki name attribute. This is the attribute to generate
# the WikiName from. 
$Foswiki::cfg{ShibLdap}{WikiNameAttributes} = 'cn';

# **BOOLEAN**
# Allow/disallow changing the LDAP password using the ChangePassword feature
$Foswiki::cfg{ShibLdap}{AllowChangePassword} = 0;

# **SELECTCLASS none,Foswiki::Users::*User**
# Define a secondary password manager used to authenticate users that are 
# registered to the wiki natively. Note, that <b>this must not be Foswiki::Users::LdapPasswdUser again!</b>
$Foswiki::cfg{ShibLdap}{SecondaryPasswordManager} = 'none';

# ---+++ Performance settings
# The following settings are used to optimize performance in your environment. Please take care.

# **NUMBER** 
# Time in seconds when cache data expires and is reloaded anew, defaults to one day.
$Foswiki::cfg{ShibLdap}{MaxCacheAge} = 86400;

# **BOOLEAN**
# Enable precaching of LDAP data. If you switch this off the LDAP users and groups will not be
# prefetched from LDAP when building a new cache. Activated by default.
$Foswiki::cfg{ShibLdap}{Precache} = 1;

# **NUMBER**
# Number of user objects to fetch in one paged result when building the username mappings;
# this is a speed optimization option, use this value with caution.
# Requires access to the 'control' LDAP extension as an LDAP client. Use '0' to disable it.
$Foswiki::cfg{ShibLdap}{PageSize} = 500; 

# **STRING 50**
# Prevent certain names from being looked up in LDAP
$Foswiki::cfg{ShibLdap}{Exclude} = 'WikiGuest, ProjectContributor, RegistrationAgent, UnknownUser, AdminGroup, NobodyGroup, AdminUser, admin, guest';

1;
