# Module of Foswiki - The Free and Open Source Wiki, http://foswiki.org/
#
# This module is a derivative of Foswiki::LoginManager::LdapApacheLogin,
# which has the following copyright. This derivative work has been funded
# through the University of Wisconsin-Milwaukee, on behalf of the LIGO
# Scientific Collaboration. Coding done by Mike Grady, Unicon, Inc., based
# on requirements provided by the funders.
#
# Copyright (C) 2007-2012 Michael Daum http://michaeldaumconsulting.com
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
package Foswiki::LoginManager::ShibLdapApacheLogin;

=begin TML

---+ Foswiki::LoginManager::ShibLdapApacheLogin

This is a simple login manager to be used when authentication is done
using Shibboleth. The expectation is that the "login name" of the user
will be provided by the Shibboleth SP through an environment variable,
with that defaulting to REMOTE_USER unless otherwise configured to be an 
alternate ENVVAR. We can also get group information from Shibboleth.
The LdapApacheLogin module was used as the base for this module, which itself
is a simple login manager to be used when authentication is done
using apache's LDAP capabilities. In addition to the normal 
<a href="%SCRIPTURLPATH{"view"}%/%SYSTEMWEB%/PerlDoc?module=Foswiki::LoginManager::ApacheLogin">Foswiki::LoginManager::ApacheLogin</a>
manager, this one adds a check to make sure the user is already cached
in <nop>ShibLdapContrib, and that its name matches onto the configured naming conventions.
If not cached, the user will be added to the cache -- but Shib users are actually
just added to an "im memory cache", given we can refresh that info from the
environment each time.

=cut

use strict;
use Assert ();
use Foswiki::LoginManager::ApacheLogin ();
use Foswiki::Contrib::ShibLdapContrib ();
use Foswiki::Sandbox ();
use Foswiki::Func ();

@Foswiki::LoginManager::ShibLdapApacheLogin::ISA = qw( Foswiki::LoginManager::ApacheLogin );

=pod

---++ writeDebug($msg) 

Static Method to write a debug messages. 

=cut

sub writeDebug {
  Foswiki::Func::writeDebug("ShibLdapApacheLogin - $_[0]\n") if $Foswiki::cfg{ShibLdap}{Debug};
}

=begin TML

---++ ClassMethod new($session)

Construct the <nop>ShibLdapApacheLogin object

=cut

sub new {
  my ($class, $session) = @_;

  my $this = bless($class->SUPER::new($session), $class);

  $this->{ShibLdap} = Foswiki::Contrib::ShibLdapContrib::getShibLdapContrib($session);
  writeDebug("constructed a new ShibLdapApacheLogin object");
  return $this;
}

=begin TML

---++ ObjectMethod loadSession()

Load the session, sanitize the login name and make sure its user information are already
cached.

=cut

sub loadSession {
  my $this = shift;

  my $authUser = $this->SUPER::loadSession(@_);

  # Explicitly untaint it as this string comes from LDAP or ENV, and all
  # such strings are (or could be) tainted.
  # Note that this "authUser" will be based on the REMOTE_USER environment
  # variable. If a different source of the "login name" has been configured
  # (different Shib-provided environment variable), and that ENV variable has
  # a value, that will be used as the source of the login name rather than
  # REMOTE_USER. If not, we'll use REMOTE_USER.
  $authUser = Foswiki::Sandbox::untaintUnchecked($authUser);

  if (defined($authUser) && $this->{ShibLdap}{excludeMap}{$authUser}) {
      writeDebug("authUser is special, don't override");
  } else {  # get the user identity from the configured environment variable
     writeDebug("shibUserVar= $this->{ShibLdap}{shibLoginEnvironmentVariable}");
     my $shibUser = '';
     $shibUser = $ENV{$this->{ShibLdap}{shibLoginEnvironmentVariable}} || '';
     writeDebug("shibUser=$shibUser");
     $authUser = $shibUser || $authUser;
     $authUser = Foswiki::Sandbox::untaintUnchecked($authUser);
  }  

  # process authUser login name
  if (defined $authUser) {
    $authUser =~ s/^\s+//o;
    $authUser =~ s/\s+$//o;
    $authUser = $this->{ShibLdap}->fromLdapCharSet($authUser);

    $authUser = lc($authUser) unless $this->{ShibLdap}{caseSensitiveLogin};
    $authUser = $this->{ShibLdap}->shibNormalizeLoginName($authUser) if $this->{ShibLdap}{normalizeLoginName};
    writeDebug("authuser=$authUser");

    unless ($this->{ShibLdap}{excludeMap}{$authUser}) {
      $this->{ShibLdap}->cacheShibUserFromPerson($authUser);
      $this->{ShibLdap}->cacheShibGroupsForPerson($authUser);
    }
  }

  return $authUser;
}

1;
